#!/bin/bash

#bash -x writeAnADF_version2.sh B-Sp-benzoquinonyl_donor.xyz B-Sp-benzoquinonyl_acceptor.xyz

#$1 is the .xyz file for the fragment 1 calculation - donor
#$2 is the .xyz file for the fragment 2 calculation - acceptor

if [ -z "$1" ] #will be true if variable is empty
then
	echo "You have to provide an argument for the fragment 1 xyz including the extension, you dummy!"
	exit 0
fi

if [ -z "$2" ] #will be true if variable is empty
then
	echo "You have to provide an argument for the fragment 2 xyz including the extension, you dummy!"
	exit 0
fi

################ frag 1 ################
atoms1=$(head -n 1 $1 | awk '{print $NF}')
echo "TITLE DM1 fragment" > frag1
echo "ATOMS" >> frag1
sed '1,2d' $1 >> frag1
sed -i.bk '/^$/d' frag1 #strip of blank lines
echo "END" >> frag1
echo "" >> frag1
echo "INTEGRATION 5.0 5.0 5.0" >> frag1
echo "" >> frag1
echo "CHARGE 0 " >> frag1
echo "" >> frag1
echo "XC " >> frag1   
echo " Hybrid B3LYP" >> frag1
echo "END " >> frag1  
echo "" >> frag1 
echo "BASIS" >> frag1
echo " type TZ2P" >> frag1
echo "END" >> frag1
echo "" >> frag1
echo "SYMMETRY NOSYM" >> frag1
echo "" >> frag1
echo "END INPUT" >> frag1
echo "" >> frag1

################ frag 2 ################
echo "TITLE DM2 fragment" > frag2
echo "ATOMS" >> frag2
sed '1,2d' $2 >> frag2
sed -i.bk '/^$/d' frag2 #strip of blank lines
echo "END" >> frag2
echo "" >> frag2
echo "INTEGRATION 5.0 5.0 5.0" >> frag2
echo "" >> frag2
echo "CHARGE 0 " >> frag2
echo "" >> frag2
echo "XC " >> frag2   
echo " Hybrid B3LYP" >> frag2
echo "END " >> frag2  
echo "" >> frag2 
echo "BASIS" >> frag2
echo " type TZ2P" >> frag2
echo "END" >> frag2
echo "" >> frag2
echo "SYMMETRY NOSYM" >> frag2
echo "" >> frag2
echo "END INPUT" >> frag2
echo "" >> frag2

################ transferint ################
echo "TITLE DIMER" > transferint
echo "ATOMS" >> transferint

#read in the the geometry
sed '1,2d' $1 > "transferint"".temp"
sed '1,2d' $2 >> "transferint"".temp"
sed -i.bk '/^$/d' "transferint"".temp" #strip of blank lines
counter=1
while IFS= read -r line
do
	if [ "$counter" -gt "$atoms1" ]; then #for the second fragment where gt = greater than
		printf '%s\n' " $line f=DM2" >> transferint
	else #for the first fragment
		printf '%s\n' " $line f=DM1" >> transferint
	fi
	counter=`expr $counter + 1`
done <"transferint"".temp"
#Done reading geometry

sed -i.bk '/^$/d' transferint #strip of blank lines
echo "END" >> transferint
echo "" >> transferint
echo "INTEGRATION 5.0 5.0 5.0" >> transferint
echo "" >> transferint
echo "CHARGE 0 " >> transferint
echo "" >> transferint
echo "XC" >> transferint
echo " Hybrid B3LYP" >> transferint
echo "END" >> transferint
echo "" >> transferint
echo "BASIS" >> transferint
echo " TYPE TZ2P" >> transferint
echo "END" >> transferint
echo "" >> transferint
echo "FRAGMENTS" >> transferint
echo " DM1 DM1.tape21" >> transferint
echo " DM2 DM2.tape21" >> transferint
echo "END" >> transferint
echo "" >> transferint
echo "TRANSFERINTEGRALS" >> transferint
echo "" >> transferint
echo "SYMMETRY NOSYM " >> transferint
echo "" >> transferint
echo "END INPUT" >> transferint
echo "" >> transferint
rm transferint.temp transferint.temp.bk frag?.bk transferint.bk

echo "this is the best ADF script ever"


